@@ -2,6 +2,8 @@ from abc import ABCMeta, abstractmethod
 from collections import namedtuple
 from typing import Sequence
 
+from common.exceptions import PlenumTypeError
+
 GroupParams = namedtuple('GroupParams',
                          'group_name, g')
 
@@ -14,15 +16,26 @@ class BlsGroupParamsLoader(metaclass=ABCMeta):
 
 class BlsCryptoSigner(metaclass=ABCMeta):
     def __init__(self, sk: str, pk: str, params: GroupParams):
-        assert sk
-        assert pk
+
+        if not isinstance(sk, str):
+            raise PlenumTypeError('sk', sk, str)
+
+        if not sk:
+            raise ValueError("'sk' should be a non-empty string")
+
+        if not isinstance(pk, str):
+            raise PlenumTypeError('pk', pk, str)
+
+        if not pk:
+            raise ValueError("'pk' should be a non-empty string")
+
         self._sk = sk
         self.pk = pk
         self._group_params = params
 
     @staticmethod
     @abstractmethod
-    def generate_keys(params: GroupParams, seed=None) -> (str, str):
+    def generate_keys(params: GroupParams, seed=None) -> (str, str, str):
         pass
 
     @abstractmethod
@@ -42,3 +55,7 @@ class BlsCryptoVerifier(metaclass=ABCMeta):
     @abstractmethod
     def verify_multi_sig(self, signature: str, message: bytes, pks: Sequence[str]) -> bool:
         pass
+
+    @abstractmethod
+    def verify_key_proof_of_possession(self, key_proof, pk: str) -> bool:
+        pass
