@@ -8,6 +8,7 @@ import sys
 if sys.version_info < (3, 5, 0):
     raise ImportError("Python 3.5.0 or later required.")
 
+import pip  # noqa
 import os   # noqa
 import importlib    # noqa
 from importlib.util import module_from_spec, spec_from_file_location    # noqa: E402
@@ -21,6 +22,21 @@ PLUGIN_LEDGER_IDS = set()
 PLUGIN_CLIENT_REQUEST_FIELDS = {}
 
 
+def find_and_load_plugin(plugin_name, plugin_root, installed_packages):
+    if plugin_name in installed_packages:
+        # TODO: Need a test for installed packages
+        plugin_name = plugin_name.replace('-', '_')
+        plugin = importlib.import_module(plugin_name)
+    else:
+        plugin_path = os.path.join(plugin_root.__path__[0],
+                                   plugin_name, '__init__.py')
+        spec = spec_from_file_location('__init__.py', plugin_path)
+        plugin = module_from_spec(spec)
+        spec.loader.exec_module(plugin)
+
+    return plugin
+
+
 def setup_plugins():
     # TODO: Should have a check to make sure no plugin defines any conflicting ledger id or request field
     global PLUGIN_LEDGER_IDS
@@ -34,20 +50,20 @@ def setup_plugins():
     except ImportError:
         raise ImportError('Incorrect plugin root {}. No such package found'.
                           format(plugin_root))
+    sys.path.insert(0, plugin_root.__path__[0])
     enabled_plugins = config.ENABLED_PLUGINS
+    installed_packages = {p.project_name: p for p in pip.get_installed_distributions()}
     for plugin_name in enabled_plugins:
-        plugin_path = os.path.join(plugin_root.__path__[0],
-                                   plugin_name, '__init__.py')
-        spec = spec_from_file_location('__init__.py', plugin_path)
-        init = module_from_spec(spec)
-        spec.loader.exec_module(init)
-        plugin_globals = init.__dict__
+        plugin = find_and_load_plugin(plugin_name, plugin_root, installed_packages)
+        plugin_globals = plugin.__dict__
+
         if 'LEDGER_IDS' in plugin_globals:
             PLUGIN_LEDGER_IDS.update(plugin_globals['LEDGER_IDS'])
         if 'CLIENT_REQUEST_FIELDS' in plugin_globals:
             PLUGIN_CLIENT_REQUEST_FIELDS.update(plugin_globals['CLIENT_REQUEST_FIELDS'])
 
     # Reloading message types since some some schemas would have been changed
+    import plenum.common.messages.fields
     import plenum.common.messages.node_messages
     import plenum.common.messages.node_message_factory
     importlib.reload(plenum.common.messages.node_messages)
