@@ -12,5 +12,5 @@ class ActionReqHandler(RequestHandler):
     def validate(self, req: Request):
         pass
 
-    def apply(self, req: Request, cons_time: int):
+    def apply(self, req: Request, cons_time: int=None):
         pass
