@@ -12,8 +12,12 @@ from dateutil import parser
 import datetime
 
 from ledger.genesis_txn.genesis_txn_file_util import genesis_txn_path
+from plenum.common.config_util import getConfig
+from plenum.common.util import get_datetime_from_ts
+from storage.kv_store_rocksdb_int_keys import KeyValueStorageRocksdbIntKeys
 from stp_core.common.constants import ZMQ_NETWORK_PROTOCOL
 from stp_core.common.log import getlogger
+from pympler import muppy, summary, asizeof
 
 
 def decode_err_handler(error):
@@ -31,6 +35,7 @@ NODE_CONTROL_CONFIG_FILE = "node_control.conf"
 INDY_NODE_SERVICE_FILE_PATH = "/etc/systemd/system/indy-node.service"
 NODE_CONTROL_SERVICE_FILE_PATH = "/etc/systemd/system/indy-node-control.service"
 NUMBER_TXNS_FOR_DISPLAY = 10
+LIMIT_OBJECTS_FOR_PROFILER = 10
 
 
 def none_on_fail(func):
@@ -38,8 +43,8 @@ def none_on_fail(func):
         try:
             return func(*args, **kwargs)
         except Exception as ex:
-            logger.debug('Validator info tool fails to '
-                         'execute {} because {}'.format(func.__name__, repr(ex)))
+            logger.display('Validator info tool fails to execute {} because {}'.
+                           format(func.__name__, repr(ex)))
             return None
 
     return wrap
@@ -47,13 +52,27 @@ def none_on_fail(func):
 
 class ValidatorNodeInfoTool:
     JSON_SCHEMA_VERSION = '0.0.1'
+    GENERAL_DB_NAME_TEMPLATE = '{node_name}_info_db'
     GENERAL_FILE_NAME_TEMPLATE = '{node_name}_info.json'
     ADDITIONAL_FILE_NAME_TEMPLATE = '{node_name}_additional_info.json'
+    VERSION_FILE_NAME_TEMPLATE = '{node_name}_version_info.json'
 
-    def __init__(self, node):
+    def __init__(self, node, config=None):
         self._node = node
+        self._config = config or getConfig()
+        self._db = None
+        self._use_db = self._config.VALIDATOR_INFO_USE_DB
         self.__name = self._node.name
         self.__node_info_dir = self._node.node_info_dir
+        self.dump_version_info()
+        if self._use_db:
+            self._db = KeyValueStorageRocksdbIntKeys(self.__node_info_dir,
+                                                     self.GENERAL_DB_NAME_TEMPLATE.format(
+                                                         node_name=self.__name.lower()))
+
+    def stop(self):
+        if self._use_db:
+            self._db.close()
 
     @property
     def info(self):
@@ -61,31 +80,37 @@ class ValidatorNodeInfoTool:
         general_info['response-version'] = self.JSON_SCHEMA_VERSION
         general_info['timestamp'] = int(time.time())
         hardware_info = self.__hardware_info
-        software_info = self.__software_info
         pool_info = self.__pool_info
         protocol_info = self.__protocol_info
         node_info = self.__node_info
+        soft_info = self.software_info
+
         if hardware_info:
             general_info.update(hardware_info)
-        if software_info:
-            general_info.update(software_info)
         if pool_info:
             general_info.update(pool_info)
         if protocol_info:
             general_info.update(protocol_info)
         if node_info:
             general_info.update(node_info)
+        if soft_info:
+            general_info.update(soft_info)
+
         return general_info
 
+    @property
+    @none_on_fail
+    def memory_profiler(self):
+        all_objects = muppy.get_objects()
+        stats = summary.summarize(all_objects)
+        return {'Memory_profiler': [l for l in summary.format_(stats, LIMIT_OBJECTS_FOR_PROFILER)]}
+
     @property
     def additional_info(self):
         additional_info = {}
         config_info = self.__config_info
-        extractions_info = self.__extractions
         if config_info:
             additional_info.update(config_info)
-        if extractions_info:
-            additional_info.update(extractions_info)
         return additional_info
 
     def _prepare_for_json(self, item):
@@ -103,24 +128,29 @@ class ValidatorNodeInfoTool:
 
     @property
     @none_on_fail
-    def __client_ip(self):
+    def client_ip(self):
         return self._node.clientstack.ha.host
 
     @property
     @none_on_fail
-    def __client_port(self):
+    def client_port(self):
         return self._node.clientstack.ha.port
 
     @property
     @none_on_fail
-    def __node_ip(self):
+    def node_ip(self):
         return self._node.nodestack.ha.host
 
     @property
     @none_on_fail
-    def __node_port(self):
+    def node_port(self):
         return self._node.nodestack.ha.port
 
+    @property
+    @none_on_fail
+    def bls_key(self):
+        return self._node.bls_bft.bls_key_register.get_key_by_name(self._node.name)
+
     @property
     @none_on_fail
     def __did(self):
@@ -151,6 +181,16 @@ class ValidatorNodeInfoTool:
     def __pool_ledger_size(self):
         return self._node.poolLedger.size if self._node.poolLedger else 0
 
+    @property
+    @none_on_fail
+    def __config_ledger_size(self):
+        return self._node.configLedger.size
+
+    @property
+    @none_on_fail
+    def __audit_ledger_size(self):
+        return self._node.auditLedger.size
+
     @property
     @none_on_fail
     def __uptime(self):
@@ -164,7 +204,10 @@ class ValidatorNodeInfoTool:
     @property
     @none_on_fail
     def __reachable_list(self):
-        return sorted(list(self._node.nodestack.conns) + [self._node.name])
+        inst_by_name = self._node.replicas.inst_id_by_primary_name
+        tupl_list = [(name, inst_by_name.get(name, None))
+                     for name in list(self._node.nodestack.conns) + [self._node.name]]
+        return sorted(tupl_list, key=lambda x: x[0])
 
     @property
     @none_on_fail
@@ -174,13 +217,38 @@ class ValidatorNodeInfoTool:
     @property
     @none_on_fail
     def __unreachable_list(self):
-        return list(set(self._node.nodestack.remotes.keys()) - self._node.nodestack.conns)
+        inst_by_name = self._node.replicas.inst_id_by_primary_name
+        tupl_list = [(name, inst_by_name.get(name, None)) for name in
+                     list(set(self._node.nodestack.remotes.keys()) - self._node.nodestack.conns)]
+        return sorted(tupl_list, key=lambda x: x[0])
 
     @property
     @none_on_fail
     def __total_count(self):
         return len(self._node.nodestack.remotes) + 1
 
+    def _get_folder_size(self, start_path):
+        total_size = 0
+        for dirpath, dirnames, filenames in os.walk(start_path):
+            for f in filenames:
+                fp = os.path.join(dirpath, f)
+                try:
+                    total_size += os.path.getsize(fp)
+                except OSError:
+                    pass
+        return total_size
+
+    @property
+    @none_on_fail
+    def node_disk_size(self):
+        nodes_data = self._get_folder_size(self._node.ledger_dir)
+
+        return {
+            "Hardware": {
+                "HDD_used_by_node": "{} MBs".format(int(nodes_data / MBs)),
+            }
+        }
+
     @property
     @none_on_fail
     def __hardware_info(self):
@@ -188,25 +256,21 @@ class ValidatorNodeInfoTool:
         ram_all = psutil.virtual_memory()
         current_process = psutil.Process()
         ram_by_process = current_process.memory_info()
-        nodes_data = psutil.disk_usage(self._node.ledger_dir)
 
         return {
             "Hardware": {
                 "HDD_all": "{} Mbs".format(int(hdd.used / MBs)),
                 "RAM_all_free": "{} Mbs".format(int(ram_all.free / MBs)),
                 "RAM_used_by_node": "{} Mbs".format(int(ram_by_process.vms / MBs)),
-                "HDD_used_by_node": "{} MBs".format(int(nodes_data.used / MBs)),
             }
         }
 
-    @property
     @none_on_fail
-    def __software_info(self):
+    def _generate_software_info(self):
         os_version = self._prepare_for_json(platform.platform())
         installed_packages = [self._prepare_for_json(pack) for pack in pip.get_installed_distributions()]
         output = self._run_external_cmd("dpkg-query --list | grep indy")
         indy_packages = output.split(os.linesep)
-
         return {
             "Software": {
                 "OS_version": os_version,
@@ -216,6 +280,21 @@ class ValidatorNodeInfoTool:
             }
         }
 
+    @property
+    @none_on_fail
+    def software_info(self):
+        file_name = self.VERSION_FILE_NAME_TEMPLATE.format(node_name=self.__name.lower())
+        path = os.path.join(self.__node_info_dir, file_name)
+        with open(path, "r") as version_file:
+            version_info = json.load(version_file)
+        return version_info
+
+    def dump_version_info(self):
+        info = self._generate_software_info()
+        file_name = self.VERSION_FILE_NAME_TEMPLATE.format(node_name=self.__name.lower())
+        path = os.path.join(self.__node_info_dir, file_name)
+        self._dump_into_file(path, info)
+
     def _cat_file(self, path_to_file):
         output = self._run_external_cmd("cat {}".format(path_to_file))
         return output.split(os.linesep)
@@ -347,16 +426,14 @@ class ValidatorNodeInfoTool:
     @none_on_fail
     def __replicas_status(self):
         res = {}
-        for replica in self._node.replicas:
+        for replica in self._node.replicas.values():
             replica_stat = {}
             replica_stat["Primary"] = self._prepare_for_json(replica.primaryName)
             replica_stat["Watermarks"] = "{}:{}".format(replica.h, replica.H)
             replica_stat["Last_ordered_3PC"] = self._prepare_for_json(replica.last_ordered_3pc)
             stashed_txns = {}
-            stashed_txns["Stashed_checkoints"] = self._prepare_for_json(len(replica.stashedRecvdCheckpoints))
-            if replica.prePreparesPendingPrevPP:
-                stashed_txns["Min_stashed_PrePrepare"] = self._prepare_for_json(
-                    [pp for pp in replica.prePreparesPendingPrevPP.itervalues()][-1])
+            stashed_txns["Stashed_checkpoints"] = self._prepare_for_json(len(replica.stashedRecvdCheckpoints))
+            stashed_txns["Stashed_PrePrepare"] = self._prepare_for_json(len(replica.prePreparesPendingPrevPP))
             replica_stat["Stashed_txns"] = stashed_txns
             res[replica.name] = self._prepare_for_json(replica_stat)
         return res
@@ -379,6 +456,8 @@ class ValidatorNodeInfoTool:
                 'transaction-count': {
                     'ledger': self.__domain_ledger_size,
                     'pool': self.__pool_ledger_size,
+                    'config': self.__config_ledger_size,
+                    'audit': self.__audit_ledger_size,
                 },
                 'uptime': self.__uptime,
             })
@@ -386,17 +465,48 @@ class ValidatorNodeInfoTool:
 
     def _get_ic_queue(self):
         ic_queue = {}
-        for view_no, queue in self._node.view_changer.instanceChanges.items():
-            ics = {}
-            ics["Vouters"] = self._prepare_for_json(list(queue.voters))
-            ics["Message"] = self._prepare_for_json(queue.msg)
-            ic_queue[view_no] = self._prepare_for_json(ics)
+        for view_no, votes in self._node.view_changer.instance_changes.items():
+            ics = {voter: {"reason": vote.reason}
+                   for voter, vote in votes.items()}
+            ic_queue[view_no] = {"Voters": self._prepare_for_json(ics)}
         return ic_queue
 
     def __get_start_vc_ts(self):
         ts = self._node.view_changer.start_view_change_ts
         return str(datetime.datetime.utcfromtimestamp(ts))
 
+    @property
+    @none_on_fail
+    def __memory_info(self):
+        # Get all memory info and get details with 20 depth
+        size_obj = asizeof.asized(self._node, detail=20)
+        whole_size = size_obj.size
+        size_obj = next(r for r in size_obj.refs if r.name == '__dict__')
+        size_dict = dict()
+        # Sort in descending order to select most 'heavy' collections
+        for num, sub_obj in enumerate(sorted(size_obj.refs, key=lambda v: v.size, reverse=True)):
+            if num > 5:
+                break
+            size_dict[sub_obj.name] = dict()
+            size_dict[sub_obj.name]['size'] = sub_obj.size
+
+            # Check if this object (which include __dict__ and __class__) or iterable (dict, list, etc ..)
+            if len(sub_obj.refs) <= 2 and any(r.name == '__dict__' for r in sub_obj.refs):
+                sub_obj_ref = next(r for r in sub_obj.refs if r.name == '__dict__')
+            else:
+                sub_obj_ref = sub_obj
+
+            for num, sub_sub_obj in enumerate(sorted(sub_obj_ref.refs, key=lambda v: v.size, reverse=True)):
+                if num > 5:
+                    break
+                size_dict[sub_obj.name][sub_sub_obj.name] = sub_sub_obj.size
+
+        result_dict = {"whole_node_size": whole_size}
+        result_dict.update(size_dict)
+        return {
+            'Memory': result_dict
+        }
+
     @property
     @none_on_fail
     def __node_info(self):
@@ -404,46 +514,61 @@ class ValidatorNodeInfoTool:
         waiting_cp = {}
         num_txns_in_catchup = {}
         last_txn_3PC_keys = {}
-        root_hashes = {}
-        uncommited_root_hashes = {}
-        uncommited_txns = {}
+        committed_ledger_root_hashes = {}
+        uncommited_ledger_root_hashes = {}
+        uncommitted_ledger_txns = {}
+        committed_state_root_hashes = {}
+        uncommitted_state_root_hashes = {}
+        freshness_status = {}
         for idx, linfo in self._node.ledgerManager.ledgerRegistry.items():
             ledger_statuses[idx] = self._prepare_for_json(linfo.state.name)
             waiting_cp[idx] = self._prepare_for_json(linfo.catchUpTill)
             num_txns_in_catchup[idx] = self._prepare_for_json(linfo.num_txns_caught_up)
             last_txn_3PC_keys[idx] = self._prepare_for_json(linfo.last_txn_3PC_key)
             if linfo.ledger.uncommittedRootHash:
-                uncommited_root_hashes[idx] = self._prepare_for_json(base58.b58encode(linfo.ledger.uncommittedRootHash))
-            uncommited_txns[idx] = [self._prepare_for_json(txn) for txn in linfo.ledger.uncommittedTxns]
+                uncommited_ledger_root_hashes[idx] = self._prepare_for_json(base58.b58encode(linfo.ledger.uncommittedRootHash))
+            txns = {"Count": len(linfo.ledger.uncommittedTxns)}
+            if len(linfo.ledger.uncommittedTxns) > 0:
+                txns["First_txn"] = self._prepare_for_json(linfo.ledger.uncommittedTxns[0])
+                txns["Last_txn"] = self._prepare_for_json(linfo.ledger.uncommittedTxns[-1])
+            uncommitted_ledger_txns[idx] = txns
             if linfo.ledger.tree.root_hash:
-                root_hashes[idx] = self._prepare_for_json(base58.b58encode(linfo.ledger.tree.root_hash))
+                committed_ledger_root_hashes[idx] = self._prepare_for_json(base58.b58encode(linfo.ledger.tree.root_hash))
+        for l_id, req_handler in self._node.ledger_to_req_handler.items():
+            committed_state_root_hashes[l_id] = self._prepare_for_json(base58.b58encode(req_handler.state.committedHeadHash))
+            uncommitted_state_root_hashes[l_id] = self._prepare_for_json(base58.b58encode(req_handler.state.headHash))
+
+        ledger_freshnesses = self._get_ledgers_updated_time() or {}
+        for idx, updated_ts in ledger_freshnesses.items():
+            freshness_status.setdefault(idx, {}).update({'Last_updated_time': self._prepare_for_json(
+                get_datetime_from_ts(updated_ts))})
+            freshness_status[idx]['Has_write_consensus'] = self._prepare_for_json(
+                self._is_updated_time_acceptable(updated_ts))
 
         return {
             "Node_info": {
-                "Name": self._prepare_for_json(
-                    self.__alias),
-                "Mode": self._prepare_for_json(
-                    self._node.mode.name),
-                "Client_port": self._prepare_for_json(
-                    self.__client_port),
-                "Client_protocol": self._prepare_for_json(
-                    ZMQ_NETWORK_PROTOCOL),
-                "Node_port": self._prepare_for_json(
-                    self.__node_port),
-                "Node_protocol": self._prepare_for_json(
-                    ZMQ_NETWORK_PROTOCOL),
-                "did": self._prepare_for_json(
-                    self.__did),
-                'verkey': self._prepare_for_json(
-                    self.__verkey),
-                "Metrics": self._prepare_for_json(
-                    self._get_node_metrics()),
-                "Root_hashes": self._prepare_for_json(
-                    root_hashes),
-                "Uncommitted_root_hashes": self._prepare_for_json(
-                    uncommited_root_hashes),
-                "Uncommitted_txns": self._prepare_for_json(
-                    uncommited_txns),
+                "Name": self._prepare_for_json(self.__alias),
+                "Mode": self._prepare_for_json(self._node.mode.name),
+                "Client_port": self._prepare_for_json(self.client_port),
+                "Client_ip": self._prepare_for_json(self.client_ip),
+                "Client_protocol": self._prepare_for_json(ZMQ_NETWORK_PROTOCOL),
+                "Node_port": self._prepare_for_json(self.node_port),
+                "Node_ip": self._prepare_for_json(self.node_ip),
+                "Node_protocol": self._prepare_for_json(ZMQ_NETWORK_PROTOCOL),
+                "did": self._prepare_for_json(self.__did),
+                'verkey': self._prepare_for_json(self.__verkey),
+                'BLS_key': self._prepare_for_json(self.bls_key),
+                "Metrics": self._prepare_for_json(self._get_node_metrics()),
+                "Committed_ledger_root_hashes": self._prepare_for_json(
+                    committed_ledger_root_hashes),
+                "Committed_state_root_hashes": self._prepare_for_json(
+                    committed_state_root_hashes),
+                "Uncommitted_ledger_root_hashes": self._prepare_for_json(
+                    uncommited_ledger_root_hashes),
+                "Uncommitted_ledger_txns": self._prepare_for_json(
+                    uncommitted_ledger_txns),
+                "Uncommitted_state_root_hashes": self._prepare_for_json(
+                    uncommitted_state_root_hashes),
                 "View_change_status": {
                     "View_No": self._prepare_for_json(
                         self._node.viewNo),
@@ -469,22 +594,23 @@ class ValidatorNodeInfoTool:
                     "Last_txn_3PC_keys": self._prepare_for_json(
                         last_txn_3PC_keys),
                 },
+                "Freshness_status": self._prepare_for_json(freshness_status),
+                "Requests_timeouts": {
+                    "Propagates_phase_req_timeouts": self._prepare_for_json(
+                        self._node.propagates_phase_req_timeouts),
+                    "Ordering_phase_req_timeouts": self._prepare_for_json(
+                        self._node.ordering_phase_req_timeouts)
+                },
                 "Count_of_replicas": self._prepare_for_json(
                     len(self._node.replicas)),
                 "Replicas_status": self._prepare_for_json(
                     self.__replicas_status),
-                "Pool_ledger_size": self._prepare_for_json(
-                    self._get_pool_ledger_size()),
-                "Domain_ledger_size": self._prepare_for_json(
-                    self._get_domain_ledger_size()),
-                "Config_ledger_size": self._prepare_for_json(
-                    self._get_config_ledger_size()),
             }
         }
 
     @property
     @none_on_fail
-    def __extractions(self):
+    def extractions(self):
         return {
             "Extractions":
                 {
@@ -538,7 +664,9 @@ class ValidatorNodeInfoTool:
                                                             self._node.config.upgradeLogFile))
             if os.path.exists(path_to_upgrade_log):
                 with open(path_to_upgrade_log, 'r') as upgrade_log:
-                    output = upgrade_log.readlines()
+                    log = upgrade_log.readlines()
+                    log_size = self._config.VALIDATOR_INFO_UPGRADE_LOG_SIZE
+                    output = log if log_size < 0 or log_size > len(log) else log[-log_size:]
         return output
 
     def _get_last_n_from_pool_ledger(self):
@@ -571,32 +699,29 @@ class ValidatorNodeInfoTool:
             i += 1
         return txns
 
-    def _get_pool_ledger_size(self):
-        return self._node.poolLedger.size
-
-    def _get_domain_ledger_size(self):
-        return self._node.domainLedger.size
-
-    def _get_config_ledger_size(self):
-        return self._node.configLedger.size
-
-    def dump_general_info(self):
-        file_name = self.GENERAL_FILE_NAME_TEMPLATE.format(node_name=self.__name.lower())
-        path = os.path.join(self.__node_info_dir, file_name)
-        with open(path, 'w') as fd:
+    def _dump_into_file(self, file_path, info):
+        with open(file_path, 'w') as fd:
             try:
-                json.dump(self.info, fd)
+                json.dump(info, fd)
             except Exception as ex:
                 logger.error("Error while dumping into json: {}".format(repr(ex)))
 
+    def _dump_into_db(self, info):
+        self._db.put(info['timestamp'], json.dumps(info))
+
+    def dump_general_info(self):
+        info = self.info
+        if self._use_db:
+            self._dump_into_db(info)
+
+        file_name = self.GENERAL_FILE_NAME_TEMPLATE.format(node_name=self.__name.lower())
+        path = os.path.join(self.__node_info_dir, file_name)
+        self._dump_into_file(path, info)
+
     def dump_additional_info(self):
         file_name = self.ADDITIONAL_FILE_NAME_TEMPLATE.format(node_name=self.__name.lower())
         path = os.path.join(self.__node_info_dir, file_name)
-        with open(path, 'w') as fd:
-            try:
-                json.dump(self.additional_info, fd)
-            except Exception as ex:
-                logger.error("Error while dumping into json: {}".format(repr(ex)))
+        self._dump_into_file(path, self.additional_info)
 
     def _get_time_from_journalctl_line(self, line):
         items = line.split(' ')
@@ -626,3 +751,11 @@ class ValidatorNodeInfoTool:
                     "total_count": len(stops),
                 }
         return res
+
+    def _get_ledgers_updated_time(self)->dict:
+        return self._node.master_replica.get_ledgers_last_update_time()
+
+    def _is_updated_time_acceptable(self, updated_time):
+        current_time = self._node.utc_epoch()
+        return current_time - updated_time <= self._node.config.ACCEPTABLE_FRESHNESS_INTERVALS_COUNT *\
+            self._node.config.STATE_FRESHNESS_UPDATE_INTERVAL
