@@ -1,23 +1,16 @@
 from logging import getLogger
 
-import pytest
-
-from plenum.common.messages.node_messages import PrePrepare, Prepare, Commit, Checkpoint
-
 from plenum.common.constants import DOMAIN_LEDGER_ID
 from plenum.server.replica import Replica
 
 from plenum.test.checkpoints.conftest import tconf, chkFreqPatched, \
     reqs_for_checkpoint
 from plenum.test.helper import send_reqs_batches_and_get_suff_replies
-from plenum.test.node_catchup.helper import waitNodeDataInequality, waitNodeDataEquality
-from plenum.test.test_node import getNonPrimaryReplicas
+from plenum.test.node_catchup.helper import waitNodeDataInequality, waitNodeDataEquality, \
+    repair_broken_node
 
 logger = getLogger()
 
-TestRunningTimeLimitSec = 200
-
-
 CHK_FREQ = 5
 LOG_SIZE = 3 * CHK_FREQ
 
@@ -80,39 +73,6 @@ def test_node_catchup_after_checkpoints(
     waitNodeDataEquality(looper, repaired_node, *other_nodes)
 
 
-@pytest.fixture
-def broken_node_and_others(txnPoolNodeSet):
-    node = getNonPrimaryReplicas(txnPoolNodeSet, 0)[-1].node
-    other = [n for n in txnPoolNodeSet if n != node]
-
-    def brokenSendToReplica(msg, frm):
-        logger.warning(
-            "{} is broken. 'sendToReplica' does nothing".format(node.name))
-
-    node.nodeMsgRouter.extend(
-        (
-            (PrePrepare, brokenSendToReplica),
-            (Prepare, brokenSendToReplica),
-            (Commit, brokenSendToReplica),
-            (Checkpoint, brokenSendToReplica),
-        )
-    )
-
-    return node, other
-
-
-def repair_broken_node(node):
-    node.nodeMsgRouter.extend(
-        (
-            (PrePrepare, node.sendToReplica),
-            (Prepare, node.sendToReplica),
-            (Commit, node.sendToReplica),
-            (Checkpoint, node.sendToReplica),
-        )
-    )
-    return node
-
-
 def get_number_of_completed_catchups(node):
     cnt = 0
     for entry in node.ledgerManager.spylog.getAll(
