@@ -1,3 +1,5 @@
+import pytest
+
 from plenum.test.delayers import delay_3pc_messages
 from plenum.test.helper import countDiscarded
 from plenum.test.node_catchup.helper import waitNodeDataEquality
@@ -8,6 +10,18 @@ from stp_core.loop.eventually import eventually
 from plenum.test.helper import sdk_send_batches_of_random_and_check
 
 
+@pytest.fixture(scope="module")
+def tconf(tconf):
+    oldMax3PCBatchSize = tconf.Max3PCBatchSize
+    oldMax3PCBatchWait = tconf.Max3PCBatchWait
+    tconf.Max3PCBatchSize = 6
+    tconf.Max3PCBatchWait = 2
+    yield tconf
+
+    tconf.Max3PCBatchSize = oldMax3PCBatchSize
+    tconf.Max3PCBatchWait = oldMax3PCBatchWait
+
+
 def test_discard_3PC_messages_for_already_ordered(looper, txnPoolNodeSet,
                                                   sdk_wallet_client, sdk_pool_handle):
     """
