@@ -44,7 +44,7 @@ def setup(txnPoolNodeSet):
 @pytest.fixture(scope="module")
 def afterElection(setup):
     for n in setup.faulties:
-        for r in n.replicas:
+        for r in n.replicas.values():
             assert not r.isPrimary
 
 
@@ -65,6 +65,6 @@ def testNumOfPrePrepareWithFPlusOneFaults(
         txnPoolNodeSet,
         preprepared1WithDelay):
     for n in txnPoolNodeSet:
-        for r in n.replicas:
+        for r in n.replicas.values():
             if r.isPrimary:
                 logger.info("{} is primary".format(r))
