@@ -53,7 +53,7 @@ def setup(txnPoolNodeSet):
 @pytest.fixture(scope="module")
 def afterElection(setup):
     for n in setup.faulties:
-        for r in n.replicas:
+        for r in n.replicas.values():
             assert not r.isPrimary
 
 
