@@ -12,7 +12,7 @@ from plenum.test.node_catchup.helper import waitNodeDataEquality
 def _get_ppseqno(nodes):
     res = set()
     for node in nodes:
-        for repl in node.replicas:
+        for repl in node.replicas.values():
             res.add(repl.lastPrePrepareSeqNo)
     assert (len(res) == 1)
     return min(res)
@@ -20,7 +20,7 @@ def _get_ppseqno(nodes):
 
 def _set_ppseqno(nodes, new_ppsn):
     for node in nodes:
-        for repl in node.replicas:
+        for repl in node.replicas.values():
             repl.lastPrePrepareSeqNo = new_ppsn
             repl.h = new_ppsn
             repl.last_ordered_3pc = (repl.viewNo, new_ppsn)
@@ -28,7 +28,7 @@ def _set_ppseqno(nodes, new_ppsn):
 
 @pytest.mark.parametrize('do_view_change', [0, 1])
 def test_add_node_to_pool_with_large_ppseqno_diff_views(do_view_change, looper, txnPoolNodeSet, tconf, sdk_pool_handle,
-                                                        sdk_wallet_steward, tdir, client_tdir, allPluginsPath):
+                                                        sdk_wallet_steward, tdir, allPluginsPath):
     """
     Adding a node to the pool while ppSeqNo is big caused a node to stash all the
     requests because of incorrect watermarks limits set.
