@@ -1,11 +1,10 @@
+from plenum.common.util import hexToFriendly
+
 from stp_core.common.log import getlogger
 
-from plenum.common.constants import ALIAS, SERVICES
+from plenum.test.pool_transactions.helper import sdk_send_update_node
 
-from plenum.test.pool_transactions.helper import updateNodeData
-
-from plenum.test.helper import checkViewNoForNodes, \
-    sendReqsToNodesAndVerifySuffReplies
+from plenum.test.helper import checkViewNoForNodes, sdk_send_random_and_check
 from plenum.test.test_node import ensureElectionsDone
 from plenum.test.view_change.helper import ensure_view_change_complete
 
@@ -13,7 +12,9 @@ logger = getlogger()
 
 
 def test_primary_selection_after_primary_demotion_and_view_changes(looper, txnPoolNodeSet,
-                                                                   stewardAndWalletForMasterNode, txnPoolMasterNodes):
+                                                                   sdk_pool_handle,
+                                                                   sdk_wallet_steward,
+                                                                   txnPoolMasterNodes):
     """
     Demote primary and do multiple view changes forcing primaries rotation.
     Demoted primary should be skipped without additional view changes.
@@ -24,12 +25,13 @@ def test_primary_selection_after_primary_demotion_and_view_changes(looper, txnPo
     logger.info("1. turn off the node which has primary replica for master instanse, "
                 " this should trigger view change")
     master_node = txnPoolMasterNodes[0]
-    client, wallet = stewardAndWalletForMasterNode
-    node_data = {
-        ALIAS: master_node.name,
-        SERVICES: []
-    }
-    updateNodeData(looper, client, wallet, master_node, node_data)
+    node_dest = hexToFriendly(master_node.nodestack.verhex)
+    sdk_send_update_node(looper, sdk_wallet_steward,
+                         sdk_pool_handle,
+                         node_dest, master_node.name,
+                         None, None,
+                         None, None,
+                         services=[])
 
     restNodes = [node for node in txnPoolNodeSet \
                  if node.name != master_node.name]
@@ -43,7 +45,8 @@ def test_primary_selection_after_primary_demotion_and_view_changes(looper, txnPo
     assert restNodes[0].replicas[0].primaryName != master_node.name
 
     # ensure pool is working properly
-    sendReqsToNodesAndVerifySuffReplies(looper, wallet, client, numReqs=3)
+    sdk_send_random_and_check(looper, txnPoolNodeSet, sdk_pool_handle,
+                              sdk_wallet_steward, 3)
 
     logger.info("2. force view change 2 and check final viewNo")
     ensure_view_change_complete(looper, restNodes)
@@ -52,7 +55,8 @@ def test_primary_selection_after_primary_demotion_and_view_changes(looper, txnPo
     assert restNodes[0].replicas[0].primaryName != master_node.name
     assert viewNo2 == viewNo1 + 1
 
-    sendReqsToNodesAndVerifySuffReplies(looper, wallet, client, numReqs=3)
+    sdk_send_random_and_check(looper, txnPoolNodeSet, sdk_pool_handle,
+                              sdk_wallet_steward, 3)
 
     logger.info("3. force view change 3 and check final viewNo")
     ensure_view_change_complete(looper, restNodes)
@@ -60,7 +64,8 @@ def test_primary_selection_after_primary_demotion_and_view_changes(looper, txnPo
     assert restNodes[0].replicas[0].primaryName != master_node.name
     assert viewNo3 == viewNo2 + 1
 
-    sendReqsToNodesAndVerifySuffReplies(looper, wallet, client, numReqs=3)
+    sdk_send_random_and_check(looper, txnPoolNodeSet, sdk_pool_handle,
+                              sdk_wallet_steward, 3)
 
     logger.info("4. force view change 4 and check final viewNo")
     ensure_view_change_complete(looper, restNodes)
@@ -68,4 +73,5 @@ def test_primary_selection_after_primary_demotion_and_view_changes(looper, txnPo
     assert restNodes[0].replicas[0].primaryName != master_node.name
     assert viewNo4 == viewNo3 + 1
 
-    sendReqsToNodesAndVerifySuffReplies(looper, wallet, client, numReqs=3)
+    sdk_send_random_and_check(looper, txnPoolNodeSet, sdk_pool_handle,
+                              sdk_wallet_steward, 3)
