@@ -9,10 +9,10 @@ from typing import Iterable, Iterator, Tuple, Sequence, Dict, TypeVar, \
     List, Optional
 
 from crypto.bls.bls_bft import BlsBft
-from plenum.common.stacks import nodeStackClass, clientStackClass
 from plenum.common.txn_util import get_from, get_req_id, get_payload_data, get_type
 from plenum.server.client_authn import CoreAuthNr
 from plenum.server.domain_req_handler import DomainRequestHandler
+from plenum.server.replica_stasher import ReplicaStasher
 from stp_core.crypto.util import randomSeed
 from stp_core.network.port_dispenser import genHa
 
@@ -33,6 +33,7 @@ from plenum.server import replica
 from plenum.server.instances import Instances
 from plenum.server.monitor import Monitor
 from plenum.server.node import Node
+from plenum.server.view_change.node_view_changer import create_view_changer
 from plenum.server.view_change.view_changer import ViewChanger
 from plenum.server.primary_elector import PrimaryElector
 from plenum.server.primary_selector import PrimarySelector
@@ -55,6 +56,7 @@ from plenum.common.messages.node_messages import Reply
 logger = getlogger()
 
 
+@spyable(methods=[CoreAuthNr.authenticate])
 class TestCoreAuthnr(CoreAuthNr):
     write_types = CoreAuthNr.write_types.union({'buy', 'randombuy'})
     query_types = CoreAuthNr.query_types.union({'get_buy', })
@@ -87,6 +89,9 @@ class TestDomainRequestHandler(DomainRequestHandler):
         else:
             super()._updateStateWithSingleTxn(txn, isCommitted=isCommitted)
 
+    def gen_txn_path(self, txn):
+        return None
+
 
 NodeRef = TypeVar('NodeRef', Node, str)
 
@@ -139,13 +144,13 @@ class TestNodeCore(StackedTester):
             nodeInfo=self.nodeInfo,
             notifierEventTriggeringConfig=notifierEventTriggeringConfig,
             pluginPaths=pluginPaths)
-        for i in range(len(self.replicas)):
-            self.monitor.addInstance()
+        for i in self.replicas.keys():
+            self.monitor.addInstance(i)
         self.replicas._monitor = self.monitor
         self.replicas.register_monitor_handler()
 
     def create_replicas(self, config=None):
-        return TestReplicas(self, self.monitor, config)
+        return TestReplicas(self, self.monitor, config, self.metrics)
 
     async def processNodeInBox(self):
         self.nodeIbStasher.process()
@@ -159,18 +164,17 @@ class TestNodeCore(StackedTester):
         self.actionQueueStasher.process()
         return super()._serviceActions()
 
-    def createReplica(self, instNo: int, isMaster: bool, config=None):
-        return TestReplica(self, instNo, isMaster, config)
-
     def newPrimaryDecider(self):
         pdCls = self.primaryDecider if self.primaryDecider else \
             TestPrimarySelector
         return pdCls(self)
 
     def newViewChanger(self):
-        vchCls = self.view_changer if self.view_changer is not None else \
-            TestViewChanger
-        return vchCls(self)
+        view_changer = self.view_changer if self.view_changer is not None \
+            else create_view_changer(self, TestViewChanger)
+        # TODO: This is a hack for tests compatibility, do something better
+        view_changer.node = self
+        return view_changer
 
     def delaySelfNomination(self, delay: Seconds):
         if isinstance(self.primaryDecider, PrimaryElector):
@@ -195,7 +199,7 @@ class TestNodeCore(StackedTester):
         logger.debug("{} resetting delays".format(self))
         self.nodestack.resetDelays()
         self.nodeIbStasher.resetDelays(*names)
-        for r in self.replicas:
+        for r in self.replicas.values():
             r.outBoxTestStasher.resetDelays()
 
     def resetDelaysClient(self):
@@ -207,7 +211,7 @@ class TestNodeCore(StackedTester):
     def force_process_delayeds(self, *names):
         c = self.nodestack.force_process_delayeds(*names)
         c += self.nodeIbStasher.force_unstash(*names)
-        for r in self.replicas:
+        for r in self.replicas.values():
             c += r.outBoxTestStasher.force_unstash(*names)
         logger.debug("{} forced processing of delayed messages, "
                      "{} processed in total".format(self, c))
@@ -272,14 +276,14 @@ class TestNodeCore(StackedTester):
                      format(self.nodestack.name, msg, frm))
 
     def service_replicas_outbox(self, *args, **kwargs) -> int:
-        for r in self.replicas:  # type: TestReplica
+        for r in self.replicas.values():  # type: TestReplica
             r.outBoxTestStasher.process()
         return super().service_replicas_outbox(*args, **kwargs)
 
     def ensureKeysAreSetup(self):
         pass
 
-    def getDomainReqHandler(self):
+    def init_domain_req_handler(self):
         return TestDomainRequestHandler(self.domainLedger,
                                         self.states[DOMAIN_LEDGER_ID],
                                         self.config, self.reqProcessors,
@@ -330,9 +334,11 @@ node_spyables = [Node.handleOneNodeMsg,
                  Node.lost_master_primary,
                  Node.propose_view_change,
                  Node.getReplyFromLedger,
+                 Node.getReplyFromLedgerForRequest,
                  Node.recordAndPropagate,
                  Node.allLedgersCaughtUp,
                  Node.start_catchup,
+                 Node._do_start_catchup,
                  Node.is_catchup_needed,
                  Node.no_more_catchups_needed,
                  Node.caught_up_for_current_view,
@@ -344,16 +350,23 @@ node_spyables = [Node.handleOneNodeMsg,
                  Node.request_propagates,
                  Node.send_current_state_to_lagging_node,
                  Node.process_current_state_message,
+                 Node.transmitToClient,
+                 Node.has_ordered_till_last_prepared_certificate,
+                 Node.on_inconsistent_3pc_state,
+                 Node.apply_stashed_reqs
                  ]
 
 
 @spyable(methods=node_spyables)
 class TestNode(TestNodeCore, Node):
+
     def __init__(self, *args, **kwargs):
+        from plenum.common.stacks import nodeStackClass, clientStackClass
         self.NodeStackClass = nodeStackClass
         self.ClientStackClass = clientStackClass
 
         Node.__init__(self, *args, **kwargs)
+        self.view_changer = create_view_changer(self, TestViewChanger)
         TestNodeCore.__init__(self, *args, **kwargs)
         # Balances of all client
         self.balances = {}  # type: Dict[str, int]
@@ -372,10 +385,11 @@ class TestNode(TestNodeCore, Node):
     def get_new_ledger_manager(self):
         return TestLedgerManager(
             self,
-            ownedByNode=True,
             postAllLedgersCaughtUp=self.allLedgersCaughtUp,
             preCatchupClbk=self.preLedgerCatchUp,
-            ledger_sync_order=self.ledger_ids
+            postCatchupClbk=self.postLedgerCatchUp,
+            ledger_sync_order=self.ledger_ids,
+            metrics=self.metrics
         )
 
     def sendRepliesToClients(self, committedTxns, ppTime):
@@ -395,6 +409,9 @@ class TestNode(TestNodeCore, Node):
     def dump_additional_info(self):
         pass
 
+    def restart_clientstack(self):
+        self.clientstack.restart()
+
 
 elector_spyables = [
     PrimaryElector.discard,
@@ -428,7 +445,8 @@ view_changer_spyables = [
     ViewChanger.sendInstanceChange,
     ViewChanger._start_view_change_if_possible,
     ViewChanger.process_instance_change_msg,
-    ViewChanger.startViewChange
+    ViewChanger.startViewChange,
+    ViewChanger.process_future_view_vchd_msg
 ]
 
 
@@ -436,6 +454,15 @@ view_changer_spyables = [
 class TestViewChanger(ViewChanger):
     pass
 
+replica_stasher_spyables = [
+    ReplicaStasher.stash
+]
+
+
+@spyable(methods=replica_stasher_spyables)
+class TestReplicaStasher(ReplicaStasher):
+    pass
+
 
 replica_spyables = [
     replica.Replica.sendPrePrepare,
@@ -446,14 +473,13 @@ replica_spyables = [
     replica.Replica.processPrePrepare,
     replica.Replica.processPrepare,
     replica.Replica.processCommit,
+    replica.Replica.process_checkpoint,
     replica.Replica.doPrepare,
     replica.Replica.doOrder,
     replica.Replica.discard,
-    replica.Replica.stashOutsideWatermarks,
     replica.Replica.revert_unordered_batches,
     replica.Replica.revert,
-    replica.Replica.can_process_since_view_change_in_progress,
-    replica.Replica.processThreePhaseMsg,
+    replica.Replica.process_three_phase_msg,
     replica.Replica._request_pre_prepare,
     replica.Replica._request_pre_prepare_for_prepare,
     replica.Replica._request_prepare,
@@ -464,6 +490,8 @@ replica_spyables = [
     replica.Replica.is_pre_prepare_time_correct,
     replica.Replica.is_pre_prepare_time_acceptable,
     replica.Replica._process_stashed_pre_prepare_for_time_if_possible,
+    replica.Replica.markCheckPointStable,
+    replica.Replica.request_propagates_if_needed,
 ]
 
 
@@ -471,6 +499,7 @@ replica_spyables = [
 class TestReplica(replica.Replica):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
+        self.stasher = TestReplicaStasher(self)
         # Each TestReplica gets it's own outbox stasher, all of which TestNode
         # processes in its overridden serviceReplicaOutBox
         self.outBoxTestStasher = \
@@ -478,8 +507,12 @@ class TestReplica(replica.Replica):
 
 
 class TestReplicas(Replicas):
+    _replica_class = TestReplica
+
     def _new_replica(self, instance_id: int, is_master: bool, bls_bft: BlsBft):
-        return TestReplica(self._node, instance_id, self._config, is_master, bls_bft)
+        return self.__class__._replica_class(self._node, instance_id,
+                                                self._config, is_master,
+                                                bls_bft, self._metrics)
 
 
 # TODO: probably delete when remove from node
@@ -632,12 +665,12 @@ class TestMonitor(Monitor):
         super().__init__(*args, **kwargs)
         self.masterReqLatenciesTest = {}
 
-    def requestOrdered(self, reqIdrs: List[Tuple[str, int]], instId: int,
-                       byMaster: bool = False):
-        durations = super().requestOrdered(reqIdrs, instId, byMaster)
+    def requestOrdered(self, reqIdrs: List[str],
+                       instId: int, requests, byMaster: bool = False) -> Dict:
+        durations = super().requestOrdered(reqIdrs, instId, requests, byMaster)
         if byMaster and durations:
-            for (identifier, reqId), duration in durations.items():
-                self.masterReqLatenciesTest[identifier, reqId] = duration
+            for key, duration in durations.items():
+                self.masterReqLatenciesTest[key] = duration
 
     def reset(self):
         super().reset()
@@ -796,8 +829,8 @@ def checkEveryProtocolInstanceHasOnlyOnePrimary(looper: Looper,
                                                 nodes: Sequence[TestNode],
                                                 retryWait: float = None,
                                                 timeout: float = None,
-                                                numInstances: int = None):
-    coro = eventually(instances, nodes, numInstances,
+                                                instances_list: Sequence[int] = None):
+    coro = eventually(instances, nodes, instances_list,
                       retryWait=retryWait, timeout=timeout)
     insts, timeConsumed = timeThis(looper.run, coro)
     newTimeout = timeout - timeConsumed if timeout is not None else None
@@ -814,7 +847,7 @@ def checkEveryNodeHasAtMostOnePrimary(looper: Looper,
                                       retryWait: float = None,
                                       customTimeout: float = None):
     def checkAtMostOnePrim(node):
-        prims = [r for r in node.replicas if r.isPrimary]
+        prims = [r for r in node.replicas.values() if r.isPrimary]
         assert len(prims) <= 1
 
     timeout = customTimeout or waits.expectedPoolElectionTimeout(len(nodes))
@@ -829,14 +862,14 @@ def checkProtocolInstanceSetup(looper: Looper,
                                nodes: Sequence[TestNode],
                                retryWait: float = 1,
                                customTimeout: float = None,
-                               numInstances: int = None):
+                               instances: Sequence[int] = None):
     timeout = customTimeout or waits.expectedPoolElectionTimeout(len(nodes))
 
     checkEveryProtocolInstanceHasOnlyOnePrimary(looper=looper,
                                                 nodes=nodes,
                                                 retryWait=retryWait,
                                                 timeout=timeout,
-                                                numInstances=numInstances)
+                                                instances_list=instances)
 
     checkEveryNodeHasAtMostOnePrimary(looper=looper,
                                       nodes=nodes,
@@ -845,7 +878,7 @@ def checkProtocolInstanceSetup(looper: Looper,
 
     primaryReplicas = {replica.instId: replica
                        for node in nodes
-                       for replica in node.replicas if replica.isPrimary}
+                       for replica in node.replicas.values() if replica.isPrimary}
     return [r[1] for r in
             sorted(primaryReplicas.items(), key=operator.itemgetter(0))]
 
@@ -854,7 +887,7 @@ def ensureElectionsDone(looper: Looper,
                         nodes: Sequence[TestNode],
                         retryWait: float = None,  # seconds
                         customTimeout: float = None,
-                        numInstances: int = None) -> Sequence[TestNode]:
+                        instances_list: Sequence[int] = None) -> Sequence[TestNode]:
     # TODO: Change the name to something like `ensure_primaries_selected`
     # since there might not always be an election, there might be a round
     # robin selection
@@ -863,7 +896,7 @@ def ensureElectionsDone(looper: Looper,
 
     :param retryWait:
     :param customTimeout: specific timeout
-    :param numInstances: expected number of protocol instances
+    :param instances_list: expected number of protocol instances
     :return: primary replica for each protocol instance
     """
 
@@ -878,7 +911,7 @@ def ensureElectionsDone(looper: Looper,
         nodes=nodes,
         retryWait=retryWait,
         customTimeout=customTimeout,
-        numInstances=numInstances)
+        instances=instances_list)
 
 
 def genNodeReg(count=None, names=None) -> Dict[str, NodeDetail]:
@@ -935,12 +968,12 @@ def timeThis(func, *args, **kwargs):
 
 
 def instances(nodes: Sequence[Node],
-              numInstances: int = None) -> Dict[int, List[replica.Replica]]:
-    numInstances = (getRequiredInstances(len(nodes))
-    if numInstances is None else numInstances)
+              instances: Sequence[int] = None) -> Dict[int, List[replica.Replica]]:
+    instances = (range(getRequiredInstances(len(nodes)))
+                 if instances is None else instances)
     for n in nodes:
-        assert len(n.replicas) == numInstances
-    return {i: [n.replicas[i] for n in nodes] for i in range(numInstances)}
+        assert len(n.replicas) == len(instances)
+    return {i: [n.replicas[i] for n in nodes] for i in instances}
 
 
 def getRequiredInstances(nodeCount: int) -> int:
