@@ -1,33 +1,17 @@
 import base58
-import os
 import pytest
 import re
 
 import time
 
-from plenum.common.constants import TXN_TYPE, GET_TXN, DATA, NODE, \
-    CURRENT_PROTOCOL_VERSION, DOMAIN_LEDGER_ID
-from plenum.common.request import Request
-from plenum.common.txn_util import get_type
-from plenum.common.types import f
-from plenum.common.util import getTimeBasedId
+from plenum.common.constants import GET_TXN
 from plenum.server.validator_info_tool import ValidatorNodeInfoTool
-from plenum.test import waits
-from plenum.test.helper import check_sufficient_replies_received, \
-    sdk_send_random_and_check
-from plenum.test.node_catchup.helper import ensureClientConnectedToNodesAndPoolLedgerSame
+from plenum.test.helper import sdk_send_random_and_check
 from plenum.test.pool_transactions.helper import disconnect_node_and_ensure_disconnected
-from plenum.test.test_client import genTestClient
 from stp_core.common.constants import ZMQ_NETWORK_PROTOCOL
-from stp_core.loop.eventually import eventually
-from plenum.server.validator_info_tool import NUMBER_TXNS_FOR_DISPLAY
-from plenum.server.node import Node
 
-TEST_NODE_NAME = 'Alpha'
-INFO_FILENAME = '{}_info.json'.format(TEST_NODE_NAME.lower())
-PERIOD_SEC = 1
 nodeCount = 5
-MAX_TIME_FOR_INFO_BUILDING = 2
+MAX_TIME_FOR_INFO_BUILDING = 3
 
 
 def test_validator_info_file_alias_field_valid(info):
@@ -35,15 +19,10 @@ def test_validator_info_file_alias_field_valid(info):
 
 
 def test_validator_info_file_bindings_field_valid(info, node):
-    # don't forget enable this check if ip comes back
-    # assert info['Node_info']['Client_ip'] == node.clientstack.ha.host
-    assert 'Client_ip' not in info['Node_info']
+    assert info['Node_info']['Client_ip'] == node.clientstack.ha.host
     assert info['Node_info']['Client_port'] == node.clientstack.ha.port
     assert info['Node_info']['Client_protocol'] == ZMQ_NETWORK_PROTOCOL
-
-    # don't forget enable this check if ip comes back
-    # assert info['Node_info']['Node_ip'] == node.nodestack.ha.host
-    assert 'Node_ip' not in info['Node_info']
+    assert info['Node_info']['Node_ip'] == node.nodestack.ha.host
     assert info['Node_info']['Node_port'] == node.nodestack.ha.port
     assert info['Node_info']['Node_protocol'] == ZMQ_NETWORK_PROTOCOL
 
@@ -52,13 +31,15 @@ def test_validator_info_file_did_field_valid(info):
     assert info['Node_info']['did'] == 'JpYerf4CssDrH76z7jyQPJLnZ1vwYgvKbvcp16AB5RQ'
 
 
+def test_validator_info_file_bls_field_valid(info, node):
+    assert info['Node_info']['BLS_key'] == node.bls_bft.bls_crypto_signer.pk
+
+
 def test_validator_info_file_response_version_field_valid(info):
     assert info['response-version'] == ValidatorNodeInfoTool.JSON_SCHEMA_VERSION
 
 
-def test_validator_info_file_timestamp_field_valid(node,
-                                                   info,
-                                                   looper):
+def test_validator_info_file_timestamp_field_valid(node, info, looper):
     looper.runFor(2)
     assert re.match('\d{10}', str(info['timestamp']))
     latest_info = node._info_tool.info
@@ -89,13 +70,21 @@ def test_validator_info_file_metrics_count_ledger_field_valid(node, info):
     assert info['Node_info']['Metrics']['transaction-count']['ledger'] == txns_num
 
 
+def test_validator_info_file_metrics_count_config_field_valid(node, info):
+    txns_num = node.configLedger.size
+    assert info['Node_info']['Metrics']['transaction-count']['config'] == txns_num
+
+
+def test_validator_info_file_metrics_count_audit_field_valid(node, info):
+    txns_num = node.auditLedger.size
+    assert info['Node_info']['Metrics']['transaction-count']['audit'] == txns_num
+
+
 def test_validator_info_file_metrics_count_pool_field_valid(info):
     assert info['Node_info']['Metrics']['transaction-count']['pool'] == nodeCount
 
 
-def test_validator_info_file_metrics_uptime_field_valid(looper,
-                                                        node,
-                                                        info):
+def test_validator_info_file_metrics_uptime_field_valid(looper, node, info):
     assert info['Node_info']['Metrics']['uptime'] > 0
     looper.runFor(2)
     latest_info = node._info_tool.info
@@ -104,7 +93,7 @@ def test_validator_info_file_metrics_uptime_field_valid(looper,
 
 def test_validator_info_file_pool_fields_valid(looper, info, txnPoolNodesLooper, txnPoolNodeSet, node):
     assert info['Pool_info']['Reachable_nodes_count'] == nodeCount
-    assert info['Pool_info']['Reachable_nodes'] == sorted(list(node.name for node in txnPoolNodeSet))
+    assert info['Pool_info']['Reachable_nodes'] == [("Alpha", 0), ("Beta", 1), ("Delta", None), ("Epsilon", None), ("Gamma", None)]
     assert info['Pool_info']['Unreachable_nodes_count'] == 0
     assert info['Pool_info']['Unreachable_nodes'] == []
     assert info['Pool_info']['Total_nodes_count'] == nodeCount
@@ -114,12 +103,13 @@ def test_validator_info_file_pool_fields_valid(looper, info, txnPoolNodesLooper,
     latest_info = node._info_tool.info
 
     assert latest_info['Pool_info']['Reachable_nodes_count'] == nodeCount - 1
-    assert latest_info['Pool_info']['Reachable_nodes'] == sorted(list(node.name for node in others))
+    assert latest_info['Pool_info']['Reachable_nodes'] == [("Alpha", 0), ("Beta", 1), ("Delta", None), ("Gamma", None)]
     assert latest_info['Pool_info']['Unreachable_nodes_count'] == 1
-    assert latest_info['Pool_info']['Unreachable_nodes'] == [txnPoolNodeSet[-1].name]
+    assert latest_info['Pool_info']['Unreachable_nodes'] == [("Epsilon", None)]
     assert latest_info['Pool_info']['Total_nodes_count'] == nodeCount
 
 
+@pytest.mark.skip(reason="info will not be included by default")
 def test_hardware_info_section(info):
     assert info['Hardware']
     assert info['Hardware']['HDD_all']
@@ -139,14 +129,18 @@ def test_node_info_section(info, node):
     assert info['Node_info']
     assert info['Node_info']['Catchup_status']
     assert info['Node_info']['Catchup_status']['Last_txn_3PC_keys']
-    assert len(info['Node_info']['Catchup_status']['Last_txn_3PC_keys']) == 3
+    assert len(info['Node_info']['Catchup_status']['Last_txn_3PC_keys']) == 4
     assert info['Node_info']['Catchup_status']['Ledger_statuses']
-    assert len(info['Node_info']['Catchup_status']['Ledger_statuses']) == 3
+    assert len(info['Node_info']['Catchup_status']['Ledger_statuses']) == 4
     assert info['Node_info']['Catchup_status']['Number_txns_in_catchup']
     # TODO uncomment this, when this field would be implemented
     # assert info['Node_info']['Catchup_status']['Received_LedgerStatus']
     assert info['Node_info']['Catchup_status']['Waiting_consistency_proof_msgs']
-    assert len(info['Node_info']['Catchup_status']['Waiting_consistency_proof_msgs']) == 3
+    assert len(info['Node_info']['Catchup_status']['Waiting_consistency_proof_msgs']) == 4
+    assert "Freshness_status" in info['Node_info']
+    for idx, fs in info['Node_info']['Freshness_status'].items():
+        assert 'Last_updated_time' in fs
+        assert 'Has_write_consensus' in fs
     assert info['Node_info']['Count_of_replicas']
     # TODO uncomment this, when this field would be implemented
     # assert info['Node_info']['Last N pool ledger txns']
@@ -154,12 +148,11 @@ def test_node_info_section(info, node):
     assert info['Node_info']['Mode']
     assert info['Node_info']['Name']
     assert info['Node_info']['Replicas_status']
-    assert info['Node_info']['Root_hashes']
-    assert info['Node_info']['Root_hashes'][0]
-    assert info['Node_info']['Root_hashes'][1]
-    assert info['Node_info']['Root_hashes'][2]
-    assert 'Uncommitted_root_hashes' in info['Node_info']
-    assert 'Uncommitted_txns' in info['Node_info']
+    assert "Committed_ledger_root_hashes" in info['Node_info']
+    assert "Committed_state_root_hashes" in info['Node_info']
+    assert "Uncommitted_ledger_root_hashes" in info['Node_info']
+    assert "Uncommitted_ledger_txns" in info['Node_info']
+    assert "Uncommitted_state_root_hashes" in info['Node_info']
     assert info['Node_info']['View_change_status']
     assert 'IC_queue'       in info['Node_info']['View_change_status']
     assert 'VCDone_queue'   in info['Node_info']['View_change_status']
@@ -167,12 +160,9 @@ def test_node_info_section(info, node):
     assert 'View_No'        in info['Node_info']['View_change_status']
     assert 'Last_complete_view_no' in info['Node_info']['View_change_status']
     assert 'Last_view_change_started_at' in info['Node_info']['View_change_status']
-    assert "Pool_ledger_size" in info["Node_info"]
-    assert "Domain_ledger_size" in info["Node_info"]
-    assert "Config_ledger_size" in info["Node_info"]
-    assert info["Node_info"]['Pool_ledger_size'] == node.poolLedger.size
-    assert info["Node_info"]['Domain_ledger_size'] == node.domainLedger.size
-    assert info["Node_info"]['Config_ledger_size'] == node.configLedger.size
+    assert info['Node_info']['Requests_timeouts']
+    assert 'Propagates_phase_req_timeouts' in info['Node_info']['Requests_timeouts']
+    assert 'Ordering_phase_req_timeouts' in info['Node_info']['Requests_timeouts']
 
 
 def test_pool_info_section(info):
@@ -205,8 +195,9 @@ def test_config_info_section(node):
     assert 'iptables_config' in info['Configuration']
 
 
+@pytest.mark.skip(reason="info will not be included by default")
 def test_extractions_section(node):
-    info = node._info_tool.additional_info
+    info = node._info_tool.info
     assert "journalctl_exceptions" in info["Extractions"]
     assert "indy-node_status" in info["Extractions"]
     assert "node-control status" in info["Extractions"]
@@ -246,55 +237,6 @@ def test_protocol_info_section(info):
     assert 'Protocol' in info
 
 
-def test_dump_additional_info(node):
-    Node.dump_additional_info(node)
-    file_name = node._info_tool.ADDITIONAL_FILE_NAME_TEMPLATE.format(node_name=node.name.lower())
-    file_path = os.path.join(node.node_info_dir, file_name)
-    assert os.path.exists(file_path)
-
-
-
-@pytest.fixture(scope='function')
-def info(node):
-    return node._info_tool.info
-
-
-@pytest.fixture(scope='module')
-def node(txnPoolNodeSet):
-    for n in txnPoolNodeSet:
-        if n.name == TEST_NODE_NAME:
-            return n
-    assert False, 'Pool does not have "{}" node'.format(TEST_NODE_NAME)
-
-
-@pytest.fixture
-def read_txn_and_get_latest_info(txnPoolNodesLooper,
-                                 client_and_wallet, node):
-    client, wallet = client_and_wallet
-
-    def read_wrapped(txn_type):
-        op = {
-            TXN_TYPE: txn_type,
-            f.LEDGER_ID.nm: DOMAIN_LEDGER_ID,
-            DATA: 1
-        }
-        req = Request(identifier=wallet.defaultId,
-                      operation=op, reqId=getTimeBasedId(),
-                      protocolVersion=CURRENT_PROTOCOL_VERSION)
-        client.submitReqs(req)
-
-        timeout = waits.expectedTransactionExecutionTime(
-            len(client.inBox))
-
-        txnPoolNodesLooper.run(
-            eventually(check_sufficient_replies_received,
-                       client, req.identifier, req.reqId,
-                       retryWait=1, timeout=timeout))
-        return node._info_tool.info
-
-    return read_wrapped
-
-
 @pytest.fixture
 def write_txn_and_get_latest_info(txnPoolNodesLooper,
                                   sdk_pool_handle,
@@ -308,21 +250,3 @@ def write_txn_and_get_latest_info(txnPoolNodesLooper,
         return node._info_tool.info
 
     return write_wrapped
-
-
-@pytest.fixture(scope="function")
-def load_latest_info(node):
-    def wrapped():
-        return node._info_tool.info
-
-    return wrapped
-
-
-@pytest.fixture
-def client_and_wallet(txnPoolNodesLooper, tdirWithClientPoolTxns, txnPoolNodeSet):
-    client, wallet = genTestClient(tmpdir=tdirWithClientPoolTxns, nodes=txnPoolNodeSet,
-                                   name='reader', usePoolLedger=True)
-    txnPoolNodesLooper.add(client)
-    ensureClientConnectedToNodesAndPoolLedgerSame(txnPoolNodesLooper, client,
-                                                  *txnPoolNodeSet)
-    return client, wallet
